/*       -*- C++ -*- */
/** \file
 ********************************************************************
 * $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/include/cexfile/attribute.h $
 * $Id: attribute.h 12 2008-02-26 15:45:35Z schlemme $
 * $Date: 2008-02-26 16:45:35 +0100 (Di, 26 Feb 2008) $
 * $Revision: 12 $
 * \author Tobias Schlemmer <keinstein@users.berlios.de>
 * $Author: schlemme $ 
 * \license LGPL
 * 
 * Attribute class.
 ********************************************************************
 */	  
#ifndef CEXFILE_ATTRIBUTE_H
#define CEXFILE_ATTRIBUTE_H

#include <set>
#include "cexfile/utility.h"


namespace cexfile {
  /// Class describing an attribute of a context
  class attribute {
  public:
    /// identifier type for references
    typedef Glib::ustring idtype;

  protected:
    /// identifier for references 
    idtype id;

    /// XML node
    const xmlpp::Node * node;

    /// Name of the attribute
    Glib::ustring name;

  public:
    /// default constructor
    attribute():id(""),node(NULL),name("") {}

    /// standard constructor
    /** \param Node XML node to be parsed 
     */
    attribute(const xmlpp::Node * Node);
    
    /// convert attribute to its name
    operator Glib::ustring () const { return name; }

    /// return the identifier of the attribute
    idtype & get_id() { return id; }

  };

  /// a set mapping identifiers to attributes
  typedef std::map<attribute::idtype,attribute> attribute_s;
  /// A set of attribute_identifiers.
  typedef std::set<attribute_s::key_type> attribute_indices;

  /// add an attribute to the set of attributes
  /** \relates attribute_s
      \relates attribute
      \param s set of attributes
      \param a attribute to be added
  */
  inline attribute_s& operator += (attribute_s& s, attribute a) 
  {
    s[a.get_id()] = a;
    return s;
  }

  inline attribute::attribute(const xmlpp::Node * Node):node(Node) 
  {
    if (!Node) 
      throw xmlpp::validity_error("Empty parent Node for attributes.");

#ifdef DEBUG
    std::cerr << "Attribute: " << Node -> get_path() << std::endl;
#endif
    
    const xmlpp::Element* nodeElement =  
      dynamic_cast<const xmlpp::Element*>(Node);
  
    if (nodeElement) {
      xmlpp::Attribute * Attr = nodeElement->get_attribute("Identifier");
      if (Attr) 
	id = Attr -> get_value();
      else throw xmlpp::validity_error("Attribute has no Identifier");
    } else 
      throw xmlpp::validity_error
	("Attribute must have an id and an Type attribute.");
    name = parse_name(Node);
  }

}


#endif /* CEXFILE_ATTRIBUTE_H */
