/** \file
 ********************************************************************
 * $URL: svn+ssh://nalw02/home/alg/schlemme/svn/cextools/trunk/textpane.cpp $
 * $Id: textpane.cpp 15 2010-06-25 19:53:27Z schlemme $
 * $Date: 2010-06-25 21:53:27 +0200 (Fr, 25. Jun 2010) $
 * $Revision: 15 $
 * \author Tobias Schlemmer <keinstein@users.berlios.de>
 * $Author: schlemme $ 
 * \license LGPL
 * 
 * PDF Plotting textpane class. 
 *
 * \addtogroup cairoplotter
 * \{
 ********************************************************************
 */	  

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "textpane.h"
#include <cmath>

using namespace cexfile;


textpane::textpane (text_s t, 
		    cexfile::point center,
		    Cairo::RefPtr<Cairo::Context> c)
{
  text = t;
  context = c;
  Cairo::FontExtents fe;
  context->get_font_extents(fe);
  double extra_space = (fe.height-fe.ascent-fe.descent)*2;

#ifdef DEBUG
  std::cerr << "Textpane Center: " << center << std::endl;
  std::cerr << "Hoehe: " << fe.ascent << std::endl
	    << "Tiefe: " << fe.descent << std::endl
	    << "Grunzeilenabstand: " << fe.height << std::endl
	    << "max-x: " << fe.max_x_advance << std::endl
	    << "max-y: " << fe.max_y_advance << std::endl
	    << "extra space: " << extra_space << std::endl;
#endif

  baselineskip = fe.height;
  width = height = x_bearing =  x_advance = y_advance = 0;
  y_bearing = -fe.ascent;

  for (text_s::const_iterator i = text.begin(); i!= text.end(); i++) {
    Cairo::TextExtents te;
    context->get_text_extents(*i, te);
#ifdef DEBUG
    std::cerr << "Breite: " << te.width << std::endl;
    std::cerr << "Hoehe:  " << te.height << std::endl;
    std::cerr << "Offset: ("<< te.x_bearing << ";" << te.y_bearing << ") " 
	      << std::endl;
    std::cerr << "Verschiebung: (" << te.x_advance << ";" << te.y_advance
	      << ")" << std::endl;
#endif
    if (te.x_bearing < x_bearing) {
      width -= te.x_bearing - x_bearing;
      x_bearing = te.x_bearing;
    }
    if (te.y_bearing < y_bearing) {
      height -= te.y_bearing - y_bearing;
      y_bearing = te.y_bearing;
    }
    if (width + x_bearing < te.width + te.x_bearing) {
      width = te.width + te.x_bearing - x_bearing;
    }
    height += baselineskip * ceil(te.height/fe.height);
#ifdef DEBUG
    std::cerr << "height change: " << te.height/fe.height << std::endl;
    std::cerr << "new height:    " << height;
#endif
  }

  cexfile::point half_dims = point (width/2,height/2);
  start = center - half_dims  - cexfile::point(x_bearing, y_bearing);

  width += fe.height/2;
  height += 4 * extra_space; 
  half_dims = point (width/2,height/2);
  area = cexfile::rect(center - half_dims, center + half_dims);
}

void textpane::plot(Cairo::RefPtr<Cairo::Context> cxt) {
  cxt->save();
  cxt->rectangle(area.get_start().get_x(),
		     area.get_start().get_y(),
		     width,
		     height);
  cxt->set_source_rgb(1,1,1);
  cxt->fill_preserve();
  cxt->set_source_rgb(0,0,0);
  cxt->stroke();
  cxt->restore();

  cxt->save();
  cxt->set_source_rgb(0,0,0);
  point startpos = start;
  for (text_s::const_iterator i = text.begin(); i != text.end(); i++) {
#ifdef DEBUG
    std::cerr << "Plotting at " << startpos
	      << " label \"" <<  *i << std::endl;
#endif
    /** \todo Examine why does \c show_text() not work here? */
    cxt->move_to(startpos.get_x(),startpos.get_y());
    cxt->text_path(*i);
    startpos += point(0,baselineskip);
  }
  cxt->fill();
  cxt->restore();
}

/** \} */
